import argparse
import os

import numpy as np


def parse_args():
    parser = argparse.ArgumentParser(description="Suite2p parameters")
    parser.add_argument("--ops", default=[], type=str, help="options")
    parser.add_argument("--db", default=[], type=str, help="options")
    parser.add_argument("--tmp", default="", type=str, help="options")
    parser.add_argument("--file", default="", type=str, help="options")
    parser.add_argument("--key", default="", type=str, help="options")
    args = parser.parse_args()

    ops_path = args.ops
    db_path = args.db
    tmp_path = args.tmp
    file_name = args.file
    data_key = args.key
    return ops_path, db_path, tmp_path, file_name, data_key


def update_paths(ops_path, db_path, tmp_path, file_name, data_key):
    if ops_path == "[]":
        from suite2p.run_s2p import default_ops

        ops = default_ops()
    else:
        ops = np.load(ops_path, allow_pickle=True).item()

    if db_path == "[]":
        db = {}
    else:
        db = np.load(db_path, allow_pickle=True).item()

    # Modify the data path to the sciCORE job temp dir
    data_path = tmp_path
    fd_path = os.path.join(tmp_path, "fd")
    print("data: %s \n fd: %s" % (data_path, fd_path))

    ops["batch_size"] = 4000
    ops["input_format"] = "h5"

    db["data_path"] = []
    db["h5py"] = file_name
    db["h5py_key"] = data_key
    db["fast_disk"] = fd_path
    # db["save_folder"] = "/scicore/home/donafl00/yamauc0000/s2p_multisession"
    print(db)

    return ops, db


def main():
    ops_path, db_path, tmp_path, file_name, data_key = parse_args()
    ops, db = update_paths(ops_path, db_path, tmp_path, file_name, data_key)

    # save the files
    np.save("ops_job.npy", ops)
    np.save("db_tmp_job.npy", db)

    # import suite2p just before it is used so it is not required
    from suite2p import run_s2p

    # run the pipeline
    ops_end = run_s2p(ops=ops, db=db)
    np.save("ops_end.npy", ops_end)



def check_caiman():
    try:
        import caiman

        version = caiman.__version__
        print(f"CaImAn version {version} installed")

    except ImportError:
        print("CaImAn not installed")
    except Exception as ex:
        print("There was an issue with CaImAn")
        raise ex

    try:
        import jobcreator

        version = jobcreator.__version__
        print(f"jobcreator version {version} installed")

    except ImportError:
        print("jobcreator not installed")
    except Exception as ex:
        print("There was an issue with jobcreator")
        raise ex



#!/usr/bin/env python
import argparse
import glob
import os
import pickle
from time import sleep

import numpy as np

from ..utils.misc import get_settings

DEFAULT_MCORR_SETTINGS = {
    "fr": 10,  # movie frame rate
    "decay_time": 0.4,  # length of a typical transient in second
    "pw_rigid": False,  # flag for performing piecewise-rigid motion correction
    "max_shifts": (5, 5),  # maximum allowed rigid shift
    "gSig_filt": (3, 3),  # size of high pass spatial filtering, used in 1p data
    "strides": (48, 48),
    "overlaps": (24, 24),
    "max_deviation_rigid": (
        3
    ),  # maximum deviation allowed for patch with respect to rigid shifts
    "border_nan": "copy",  # replicate values along the boundaries
}

DEFAULT_CNMF_PARAMETERS = {
    "method_init": "corr_pnr",  # use this for 1 photon
    "K": None,  # upper bound on number of components per patch, in general None
    "gSig": (3, 3),  # gaussian width of a 2D gaussian kernel
    "gSiz": (13, 13),  # average diameter of a neuron, in general 4*gSig+1
    "merge_thr": 0.7,  # merging threshold, max correlation allowed
    "p": 1,  # order of the autoregressive system
    "tsub": 2,  # downsampling factor in time for initialization
    "ssub": 1,  # downsampling factor in space for initialization,
    "rf": 40,  # half-size of the patches in pixels
    "stride": 20,  # amount of overlap between the patches in pixels
    "only_init": True,  # set it to True to run CNMF-E
    "nb": 0,  # number of background components (rank) if positive
    "nb_patch": 0,  # number of background components (rank) per patch if gnb>0
    "method_deconvolution": "oasis",  # could use 'cvxpy' alternatively
    "low_rank_background": None,
    "update_background_components": True,
    # sometimes setting to False improve the results
    "min_corr": 0.8,  # min peak value from correlation image
    "min_pnr": 10,  # min peak to noise ration from PNR image
    "normalize_init": False,  # just leave as is
    "center_psf": True,  # leave as is for 1 photon
    "ssub_B": 2,  # additional downsampling factor in space for background
    "ring_size_factor": 1.4,  # radius of ring is gSiz*ring_size_factor,
    "del_duplicates": True,
    "border_pix": 0,
}

DEFAULT_QC_PARAMETERS = {
    "min_SNR": 2.5,  # adaptive way to set threshold on the transient size
    "rval_thr": 0.85,
    "use_cnn": False,
}


def parse_args():
    parser = argparse.ArgumentParser(description="Suite2p parameters")
    parser.add_argument("--file", default=[], type=str, help="options")
    parser.add_argument("--ncpus", default=1, type=int, help="options")
    parser.add_argument("--mc_settings", default="", type=str, help="options")
    parser.add_argument("--cnmf_settings", default="", type=str, help="options")
    parser.add_argument("--qc_settings", default="", type=str, help="options")
    args = parser.parse_args()

    file_path = args.file
    n_cpus = args.ncpus
    mc_settings = args.mc_settings
    cnmf_settings = args.cnmf_settings
    qc_settings = args.qc_settings

    return file_path, n_cpus, mc_settings, cnmf_settings, qc_settings


def run(
    file_path,
    n_cpus,
    mc_settings: dict = {},
    cnmf_settings: dict = {},
    qc_settings: dict = {},
):

    # we import the pipeline upon running so they aren't required for all installs
    import caiman as cm
    from caiman.motion_correction import MotionCorrect
    from caiman.source_extraction.cnmf import params as params
    from caiman.source_extraction import cnmf

    # load and update the pipeline settings
    mc_parameters = DEFAULT_MCORR_SETTINGS
    for k, v in mc_settings.items():
        mc_parameters[k] = v
    cnmf_parameters = DEFAULT_CNMF_PARAMETERS
    for k, v in cnmf_settings.items():
        cnmf_parameters[k] = v
    qc_parameters = DEFAULT_QC_PARAMETERS
    for k, v in qc_settings.items():
        qc_parameters[k] = v

    # get the filenames
    file_pattern = os.path.join(file_path, "*.tif")
    fnames = glob.glob(file_pattern)
    print(fnames)
    mc_parameters["fnames"] = fnames

    opts = params.CNMFParams(params_dict=mc_parameters)

    print("starting server")
    # start the server

    # n_processes = n_cpus - 1
    c, dview, n_processes = cm.cluster.setup_cluster(
        backend="local", n_processes=None, single_thread=False
    )
    print(n_processes)
    sleep(30)

    print("motion corr")
    pw_rigid = mc_parameters["pw_rigid"]
    mc = MotionCorrect(fnames, dview=dview, **opts.get_group("motion"))
    mc.motion_correct(save_movie=True)
    fname_mc = mc.fname_tot_els if pw_rigid else mc.fname_tot_rig
    if pw_rigid:
        bord_px = np.ceil(
            np.maximum(
                np.max(np.abs(mc.x_shifts_els)), np.max(np.abs(mc.y_shifts_els))
            ).astype(np.int)
        )

    print("writing mmap")
    bord_px = 0
    fname_new = cm.save_memmap(
        fname_mc, base_name="memmap_", order="C", border_to_0=bord_px
    )

    # load mmap
    print("loading mmap")

    # load memory mappable file
    Yr, dims, T = cm.load_memmap(fname_new)
    images = Yr.T.reshape((T,) + dims, order="F")

    # Don't seed with predetermined binary masks
    Ain = None

    opts.change_params(
        params_dict=cnmf_parameters
    )  # number of pixels to not consider in the borders)

    cnm = cnmf.CNMF(n_processes=n_processes, dview=dview, Ain=Ain, params=opts)
    cnm.fit(images)

    print("evaluate components")
    cnm.params.set("quality", qc_parameters)
    cnm.estimates.evaluate_components(images, cnm.params, dview=dview)

    print("saving results")
    cnm.save(cnm.mmap_file[:-4] + "hdf5")

    # save the parameters in the same dir as the results
    final_params = cnm.params.to_dict()
    path_base = os.path.dirname(cnm.mmap_file)
    params_file = os.path.join(path_base, "all_caiman_parameters.pkl")
    with open(params_file, "wb") as fp:
        pickle.dump(final_params, fp)

    print("stopping server")
    cm.stop_server(dview=dview)


def main():
    (
        file_path,
        n_cpus,
        mc_settings_path,
        cnmf_settings_path,
        qc_settings_path,
    ) = parse_args()

    mc_settings = get_settings(mc_settings_path)
    cnmf_settings = get_settings(cnmf_settings_path)
    qc_settings = get_settings(qc_settings_path)

    # run the pipeline
    run(
        file_path=file_path,
        n_cpus=n_cpus,
        mc_settings=mc_settings,
        cnmf_settings=cnmf_settings,
        qc_settings=qc_settings,
    )



import os


def caiman_job_file(
    data_path: str,
    jobcreator_output_dir: str,
    job_name: str,
    email: str,
    n_cpu: int = 8,
    mem_per_cpu: int = 25,
    tmp_size: int = 150,
    job_time: str = "04:00:00",
    qos: str = "6hours",
    log_file: str = "myrun.o",
    error_file: str = "myrun.e",
    mc_settings_file: str = "default",
    cnmf_settings_file: str = "default",
    qc_settings_file: str = "default",
):
    mem_per_cpu = str(mem_per_cpu) + "G"
    tmp_size = str(tmp_size) + "G"

    # get the name of the file and make the path to the temp dir
    data_pattern = os.path.join(data_path, "*.tif*")

    # save the error and log files to the results dir
    log_file = os.path.join(jobcreator_output_dir, log_file)
    error_file = os.path.join(jobcreator_output_dir, error_file)

    # stub for naming the environment file
    env_file_stub = os.path.join(jobcreator_output_dir, "job_")

    job_file = f"""#!/bin/bash

#SBATCH --job-name={job_name}                   #This is the name of your job
#SBATCH --cpus-per-task={n_cpu}                  #This is the number of cores reserved
#SBATCH --mem-per-cpu={mem_per_cpu}              #This is the memory reserved per core.
#SBATCH --tmp={tmp_size}

#SBATCH --time={job_time}        #This is the time that your task will run
#SBATCH --qos={qos}           #You will run in this queue

# Paths to STDOUT or STDERR files should be absolute or relative to current working directory
#SBATCH --output={log_file}%j     #These are the STDOUT and STDERR files
#SBATCH --error={error_file}%j
#SBATCH --mail-type=END,FAIL,TIME_LIMIT
#SBATCH --mail-user={email}        #You will be notified via email when your task ends or fails


#Remember:
#The variable $TMPDIR points to the local hard disks in the computing nodes.
#The variable $HOME points to your home directory.
#The variable $SLURM_JOBID stores the ID number of your job.


#load your required modules below
#################################
module purge
module load Anaconda3

#add your command lines below
#############################
echo "moving files"
for file in {data_pattern}; do cp "$file" $TMP;done

echo "analysis"
source activate caiman
conda env export > {env_file_stub}$SLURM_JOBID_env.yml

caiman_runner --file $TMP --ncpus {n_cpu} --mc_settings {mc_settings_file} --cnmf_settings {cnmf_settings_file} --qc_settings {qc_settings_file}

for file in $TMP/*.mmap; do cp "$file" {jobcreator_output_dir};done
for file in $TMP/*.hdf5; do cp "$file" {jobcreator_output_dir};done
for file in $TMP/*.pkl; do cp "$file" {jobcreator_output_dir};done
"""

    return job_file



import argparse
import json
import os
import subprocess

from ._job_templates import suite2p_job_file, caiman_job_file
from ._pipeline_checks import check_caiman

JOB_FILE_GENERATORS = {"suite2p": suite2p_job_file, "caiman": caiman_job_file}
PIPELINE_CHECKERS = {"caiman": check_caiman}


def parse_args():
    parser = argparse.ArgumentParser(description="Pipeline parameters")
    parser.add_argument("--pipeline", default="", type=str, help="options")
    parser.add_argument("--settings", default="", type=str, help="options")
    parser.add_argument("--output", default=".", type=str, help="options")
    parser.add_argument("--test", action="store_true")

    args = parser.parse_args()

    pipeline_name = args.pipeline
    settings_path = args.settings
    output_dir = args.output
    test = args.test

    return pipeline_name, settings_path, output_dir, test


def main():
    pipeline_name, settings_path, output_dir, test = parse_args()

    if test is True:
        checker_function = PIPELINE_CHECKERS[pipeline_name]
        checker_function()
    else:
        # make the output directory if it doesn't exist
        if not os.path.exists(output_dir):
            os.makedirs(output_dir)

        job_file_path = os.path.join(output_dir, "job_file.sh")

        with open(settings_path, "r") as read_file:
            job_settings = json.load(read_file)

        # add the output directory to the job file settings
        job_settings["jobcreator_output_dir"] = output_dir

        # get the file text
        job_file_gen = JOB_FILE_GENERATORS[pipeline_name]
        job_file_text = job_file_gen(**job_settings)

        # make the file job file
        with open(job_file_path, "w") as text_file:
            text_file.write(job_file_text)

        # run the job
        subprocess.run(["sbatch", job_file_path])



import os


def suite2p_job_file(
    data_path: str,
    data_key: str,
    job_name: str,
    email: str,
    jobcreator_output_dir: str,
    ops_path: str = None,
    db_path: str = None,
    n_cpu: int = 8,
    mem_per_cpu: int = 20,
    tmp_size: int = 50,
    job_time: str = "01:00:00",
    qos: str = "6hours",
    log_file: str = "myrun.o",
    error_file: str = "myrun.e",
):
    mem_per_cpu = str(mem_per_cpu) + "G"
    tmp_size = str(tmp_size) + "G"

    # get the name of the file and make the path to the temp dir
    file_name = os.path.basename(data_path)
    temp_data_path = os.path.join("$TMPDIR", file_name)

    if ops_path is None:
        ops_path = "[]"

    if db_path is None:
        db_path = "[]"

    job_file = f"""#!/bin/bash

#SBATCH --job-name={job_name}                   #This is the name of your job
#SBATCH --cpus-per-task={n_cpu}                  #This is the number of cores reserved
#SBATCH --mem-per-cpu={mem_per_cpu}              #This is the memory reserved per core.
#SBATCH --tmp={tmp_size}

#SBATCH --time={job_time}        #This is the time that your task will run
#SBATCH --qos={qos}           #You will run in this queue

# Paths to STDOUT or STDERR files should be absolute or relative to current working directory
#SBATCH --output={log_file}%j     #These are the STDOUT and STDERR files
#SBATCH --error={error_file}%j
#SBATCH --mail-type=END,FAIL,TIME_LIMIT
#SBATCH --mail-user={email}        #You will be notified via email when your task ends or fails


#Remember:
#The variable $TMPDIR points to the local hard disks in the computing nodes.
#The variable $HOME points to your home directory.
#The variable $SLURM_JOBID stores the ID number of your job.


#load your required modules below
#################################
module purge
module load Anaconda3

#add your command lines below
#############################
mkdir $TMPDIR/fd
cp {data_path} {temp_data_path}
val=$(ls $TMPDIR)
echo $val
echo $TMPDIR

source activate suite2p
conda env export > job_$SLURM_JOBID_env.yml
suite2p_runner --tmp $TMPDIR --ops {ops_path} --db {db_path} --file {temp_data_path} --key {data_key}

val=$(ls $TMPDIR)
echo $val
# for file in $TMP/*.npy; do cp "$file" {jobcreator_output_dir};done
for file in $TMP/suite2p/plane0/*.npy; do cp "$file" {jobcreator_output_dir};done
"""

    return job_file



import os

from setuptools import find_packages, setup
import versioneer

install_requires = [
    line.rstrip()
    for line in open(os.path.join(os.path.dirname(__file__), "requirements.txt"))
]

setup(
    name="jobcreator",
    install_requires=install_requires,
    version=versioneer.get_version(),
    cmdclass=versioneer.get_cmdclass(),
    description="CLI for setting up and running slurm jobs",
    url="https://github.com/donatolab/jobcreator",
    license="GPL",
    packages=find_packages(),
    entry_points={
        "console_scripts": [
            "jobcreator=jobcreator.__main__:main",
            "suite2p_runner=jobcreator._pipeline_runners.suite2p_runner:main",
            "caiman_runner=jobcreator._pipeline_runners.caiman_runner:main",
        ]
    },
    zip_safe=False,
)



import subprocess


def capture(command):
    proc = subprocess.Popen(command, stdout=subprocess.PIPE, stderr=subprocess.PIPE,)
    out, err = proc.communicate()
    return out, err, proc.returncode


def test_run_cli():
    command = ["jobcreator"]
    _, _, exitcode = capture(command)

    assert exitcode == 1



import json


def get_settings(settings_path: str) -> dict:
    """helper function to load settings"""
    if settings_path == "default":
        settings = {}

    else:
        with open(settings_path, "r") as read_file:
            settings = json.load(read_file)

    return settings
